package com.xhkjedu.smarking.service.scan;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.smarking.mapper.scan.MsScanbatchMapper;
import com.xhkjedu.smarking.mapper.scan.MsScanbatchPaperBlockMapper;
import com.xhkjedu.smarking.mapper.scan.MsScanbatchPaperBlockQuestionMapper;
import com.xhkjedu.smarking.mapper.scan.MsScanbatchPaperMapper;
import com.xhkjedu.smarking.mapper.stupaper.MsPaperStudentBlockMapper;
import com.xhkjedu.smarking.mapper.stupaper.MsPaperStudentMapper;
import com.xhkjedu.smarking.mapper.stupaper.MsPaperStudentQuestionMergeMapper;
import com.xhkjedu.smarking.mapper.system.UserMapper;
import com.xhkjedu.smarking.model.scan.MsScanbatchPaper;
import com.xhkjedu.smarking.model.scan.MsScanbatchPaperBlock;
import com.xhkjedu.smarking.model.scan.MsScanbatchPaperBlockQuestion;
import com.xhkjedu.smarking.model.stupaper.MsPaperStudent;
import com.xhkjedu.smarking.model.stupaper.MsPaperStudentBlock;
import com.xhkjedu.smarking.model.stupaper.MsPaperStudentQuestionMerge;
import com.xhkjedu.smarking.utils.ConvertUtil;
import com.xhkjedu.smarking.utils.MarkingUtil;
import com.xhkjedu.utils.N_Utils;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @Description 批阅扫描试卷表 服务实现类
 * @Author auto
 * @Date 2024-10-15
 */
@Service
public class MsScanbatchPaperService {
    @Resource
    private MsScanbatchPaperMapper msScanbatchPaperMapper;
    @Resource
    private MsScanbatchPaperBlockMapper msScanbatchPaperBlockMapper;
    @Resource
    private MsScanbatchPaperBlockQuestionMapper msScanbatchPaperBlockQuestionMapper;
    @Resource
    private MsScanbatchMapper msScanbatchMapper;
    @Resource
    private UserMapper userMapper;
    @Resource
    private MsPaperStudentMapper msPaperStudentMapper;
    @Resource
    private MsPaperStudentQuestionMergeMapper msPaperStudentQuestionMergeMapper;
    @Resource
    private MsPaperStudentBlockMapper msPaperStudentBlockMapper;
    @Resource
    private ConvertUtil convertUtil;

    /**
     * @Description 新增
     * @Param [msScanbatchPaper]
     * @Author auto
     * @Date 2024-10-15
     **/
    @Transactional(rollbackFor = Exception.class)
    public void add(MsScanbatchPaper msScanbatchPaper){
        //保存一次，扫描次数加1
        msScanbatchMapper.updateScannum(msScanbatchPaper.getBatchid());

        Integer timestamp = N_Utils.getSecondTimestamp();
        msScanbatchPaper.setCreatetime(timestamp);

        //获取考号
        String examno = msScanbatchPaper.getExamno();
        //考号不存在，异常信息2考号有误（异常类型1扫描出错 2考号有误 3考号重复 4缺考异常）
        if(N_Utils.isEmpty(examno)){
            msScanbatchPaper.setHaserror(1);
            msScanbatchPaper.setErrortype(2);
            savePaperBlockQuestions(msScanbatchPaper);
            return;
        }

        //考号存在，获取对应的试卷信息是否有异常，如果有则该条数据也标识为异常,异常信息3考号重复
        Integer errorNum = msScanbatchPaperMapper.getErrorNumByMpidAndExamno(msScanbatchPaper.getMpid(),examno);
        if(errorNum > 0){
            msScanbatchPaper.setHaserror(1);
            msScanbatchPaper.setErrortype(3);
            savePaperBlockQuestions(msScanbatchPaper);
        }

        //根据考号获取学生
        Integer userid = userMapper.getUseridByExamno(examno);
        //用户不存在，异常信息2考号有误
        if(!N_Utils.isTrueInteger(userid)){
            msScanbatchPaper.setHaserror(1);
            msScanbatchPaper.setErrortype(2);
            savePaperBlockQuestions(msScanbatchPaper);
            return;
        }
        //用户存在，获取用户试卷信息
        MsPaperStudent msPaperStudent = msPaperStudentMapper.getPaperStudentByStudentidAndMpid(userid,msScanbatchPaper.getMpid());
        //试卷不存在，异常信息1扫描出错
        if(msPaperStudent == null){
            msScanbatchPaper.setHaserror(1);
            msScanbatchPaper.setErrortype(1);
            savePaperBlockQuestions(msScanbatchPaper);
            return;
        }
        //试卷存在，判断学生是否已绑定扫描试卷信息，已有保存扫描信息本次异常，异常信息3考号重复
        if(N_Utils.isTrueInteger(msPaperStudent.getMspid())){
            msScanbatchPaper.setHaserror(1);
            msScanbatchPaper.setErrortype(3);
            savePaperBlockQuestions(msScanbatchPaper);
            //处理上一次对应扫描试卷为3考号重复，并清空学生试卷试题信息
            msScanbatchPaperMapper.updatePaperErrortype(msPaperStudent.getMspid(),3);
            updatePaperStudentQuestion(msPaperStudent);
            return;
        }


        //保存扫描信息
        savePaperBlockQuestions(msScanbatchPaper);
        //学生试卷信息
        int answernum = saveStudentBlockQuestions(msScanbatchPaper,msPaperStudent.getMpsid());
        if (!N_Utils.isEmptyInteger(msScanbatchPaper.getMissexam()) && msScanbatchPaper.getMissexam() == 1 && answernum>0) {//缺考
            //如果标记考试缺考，且答题数量大于0属于异常,异常信息4缺考异常
            msScanbatchPaper.setHaserror(1);
            msScanbatchPaper.setErrortype(4);
            //修改扫描信息
            msScanbatchPaperMapper.updatePaperErrortype(msPaperStudent.getMspid(),4);
        }
    }

    //保存扫描试卷-保存扫描题块试题信息
    private void savePaperBlockQuestions(MsScanbatchPaper msScanbatchPaper){
        msScanbatchPaperMapper.insertUseGeneratedKeys(msScanbatchPaper);
        Integer mspid = msScanbatchPaper.getId();
        msScanbatchPaper.setMspid(mspid);

        //保存试题
        List<MsScanbatchPaperBlock> spblocks = msScanbatchPaper.getSpblocks();
        for (MsScanbatchPaperBlock spblock : spblocks) {
            spblock.setMspid(mspid);
            spblock.setBatchid(msScanbatchPaper.getBatchid());
        }

        msScanbatchPaperBlockMapper.insertList(spblocks);
        List<MsScanbatchPaperBlockQuestion> saveQuestions = new ArrayList<>();
        for (MsScanbatchPaperBlock spblock : spblocks) {
            Integer spbid = spblock.getId();
            List<MsScanbatchPaperBlockQuestion> spbquestions = spblock.getSpbquestions();
            for (MsScanbatchPaperBlockQuestion spbquestion : spbquestions) {
                spbquestion.setMspbid(spbid);
                saveQuestions.add(spbquestion);
            }
        }
        if(N_Utils.isListNotEmpty(saveQuestions)){
            msScanbatchPaperBlockQuestionMapper.insertList(saveQuestions);
        }
        //更改识别数量
        msScanbatchMapper.updateIdentifynum(msScanbatchPaper.getBatchid());

    }

    //保存扫描试卷-清空学生试卷题块试题信息
    private void updatePaperStudentQuestion(MsPaperStudent msPaperStudent){
        Integer mspid = msPaperStudent.getMspid();
        //清空学生题块、试题 作答信息及批阅信息
        msPaperStudentBlockMapper.updateStudentBlockQuestionByMpsid(mspid);

        //清空学生试卷信息
        MsPaperStudent ps = new MsPaperStudent();
        ps.setBatchid(null);
        ps.setMspid(null);
        ps.setSstate(0);
        ps.setChecked(0);
        ps.setStuscore(0.0);
        ps.setStuscoretype(0);
        ps.setHasbad(0);
        ps.setPaperfront(null);
        ps.setPaperback(null);
        ps.setPagenum(0);
        ps.setMpsid(msPaperStudent.getMpsid());

        msPaperStudentMapper.updateByPrimaryKeySelective(ps);
    }

    //保存扫描试卷-保存学生试卷题块信息
    private int saveStudentBlockQuestions(MsScanbatchPaper msScanbatchPaper,Integer mpsid){
        int rtnanswernum = 0;//试题数量

        //扫描试卷中切割题块
        List<MsScanbatchPaperBlock> spblocks = msScanbatchPaper.getSpblocks();

        //学生题块试题信息
        List<MsPaperStudentBlock> paperQuestions =msPaperStudentBlockMapper.listPaperStuBlockQuestions(mpsid);
        //扫描的题块数量和学生试题题块数量不一致
        if(spblocks.size() != paperQuestions.size()){
            return -1;
        }

        Integer checknum = 0;//记录试卷中已批改题块数量
        Double stuPaperScore = 0.0;//学生试卷得分
        List<MsPaperStudentBlock>  saveBlockList = new ArrayList<>();// 保存学生题块信息
        List<MsPaperStudentQuestionMerge> saveQuesList = new ArrayList<>();//保存学生提交试题
        List<MsPaperStudentBlock> needConvertList = new ArrayList<>();//需要合并图片的题块
        Integer timestamp = N_Utils.getSecondTimestamp();

        for(MsScanbatchPaperBlock spblock : spblocks){
            String picPath = spblock.getPicpath();
            MsPaperStudentBlock studentBlock = paperQuestions.stream().filter(b -> b.getMblockid().equals(spblock.getMblockid())).findFirst().orElse(null);
            studentBlock.setStuanswer(picPath);
            List<String> stuPicList = JSON.parseArray(picPath, String.class);//判断有几张图片
            Integer converted = 1;
            if(stuPicList!= null && stuPicList.size() > 1){
                converted = 0;
            }
            studentBlock.setConverted(0);
            saveBlockList.add(studentBlock);
            needConvertList.add(studentBlock);

            List<MsPaperStudentQuestionMerge> stuQuestions = studentBlock.getQuestions();//学生试题
            List<MsScanbatchPaperBlockQuestion> spbquestions = spblock.getSpbquestions();//扫描题块下试题
            if(studentBlock.getBlocktype() == 1){
                //说明是客观题题块,需要自动批改

                for(MsScanbatchPaperBlockQuestion spbquestion : spbquestions) {
                    //试题答案
                    MsPaperStudentQuestionMerge queAnswer = stuQuestions.stream().filter(q -> q.getMbqid().equals(spbquestion.getMbqid())).findFirst().orElse(null);

                    //扫描结果
                    MsPaperStudentQuestionMerge stuQuestion = new MsPaperStudentQuestionMerge();
                    stuQuestion.setStuanswer(spbquestion.getIdentifycont());
                    stuQuestion.setMbqid(spbquestion.getMbqid());
                    stuQuestion.setMpsqmid(queAnswer.getMpsqmid());
                    stuQuestion.setAnswertime(timestamp);
                    stuQuestion.setConverted(1);
                    stuQuestion.setAnswertype(1);
                    MarkingUtil.checkObjectiveQuestion(stuQuestion, queAnswer);
                    if(stuQuestion.getChecked() == 2){
                        checknum++;
                    }
                    stuPaperScore = MarkingUtil.add(stuPaperScore, stuQuestion.getStuscore());

                    saveQuesList.add(stuQuestion);
                }
            }else{
                //主观题
                for(MsScanbatchPaperBlockQuestion spbquestion : spbquestions){
                    //试题答案
                    MsPaperStudentQuestionMerge queAnswer = stuQuestions.stream().filter(q -> q.getMbqid().equals(spbquestion.getMbqid())).findFirst().orElse(null);
                    MsPaperStudentQuestionMerge stuQuestion = new MsPaperStudentQuestionMerge();
                    stuQuestion.setMpsqmid(queAnswer.getMpsqmid());
                    stuQuestion.setStuanswer(picPath);
                    stuQuestion.setAnswertype(2);
                    stuQuestion.setAnswered(1);
                    stuQuestion.setAnswertime(timestamp);
                    stuQuestion.setConverted(converted);
                    //如果多图，需要合并
                    saveQuesList.add(stuQuestion);
                }
            }
            //保存需要合并的试题信息，合并后保存
        }

        //处理试卷信息
        MsPaperStudent msPaperStudent = new MsPaperStudent();
        msPaperStudent.setMpsid(mpsid);
        msPaperStudent.setEndtime(timestamp);
        msPaperStudent.setBatchid(msScanbatchPaper.getBatchid());
        msPaperStudent.setMspid(msScanbatchPaper.getMspid());
        msPaperStudent.setCosttime(0);
        msPaperStudent.setPaperfront(msScanbatchPaper.getPaperfront());
        msPaperStudent.setPaperback(msScanbatchPaper.getPaperback());
        msPaperStudent.setPagenum(msScanbatchPaper.getPagenum());
        msPaperStudent.setStuscore(stuPaperScore);
        msPaperStudent.setStuscoretype(1);
        if(msScanbatchPaper.getMissexam() == 1){
            //缺考
            msPaperStudent.setSstate(3);//状态0未进场1已进场2已交卷3缺考
        }else{
            msPaperStudent.setSstate(2);//状态0未进场1已进场2已交卷3缺考
        }
        if(msScanbatchPaper.getViolated() == 1){
            msPaperStudent.setHasbad(1);
        }
        if(checknum == saveQuesList.size()){
            msPaperStudent.setChecked(2);
        }

        //批量修改试卷题块、试题信息
        msPaperStudentBlockMapper.updateBatchStuBlock(saveBlockList);
        msPaperStudentQuestionMergeMapper.updateBatchStuQues(saveQuesList);
        msPaperStudentMapper.updateByPrimaryKeySelective(msPaperStudent);
        handleStudentNeedConvert(needConvertList);

        rtnanswernum = saveQuesList.size();

        return rtnanswernum;
    }

    private void handleStudentNeedConvert(List<MsPaperStudentBlock> needConvertList){
        if(needConvertList.size() > 0){
            for(MsPaperStudentBlock studentBlock : needConvertList){
                String stuAnswer = studentBlock.getStuanswer();
                List<String> stuPicList = JSON.parseArray(stuAnswer, String.class);
                if(stuPicList != null && stuPicList.size() > 1){
                    convertUtil.imgMerge(studentBlock.getMpsbid(),stuPicList,2);
                }
            }
        }
    }


    /*
     * @Description 扫描记录-查看原卷
     * @Date 2024/10/25 17:22:45
     * @Author WN
     * @Param [mspid]
     * @Return java.util.Map<java.lang.String,java.lang.Object>
     **/
    public Map<String,Object> getScanPaperQuestion(Integer mspid){
        Map<String,Object> map = msScanbatchPaperMapper.getScanPaperByMspid(mspid);
        //获取试卷中客观题
        List<Map<String,Object>> objectives = msScanbatchPaperBlockMapper.listObjectiveQuestions(mspid);
        map.put("objectives",objectives);
        //获取试卷中主观题
        List<Map<String,Object>> subjectives = msScanbatchPaperBlockMapper.listSubjectiveQuestions(mspid);
        map.put("subjectives",subjectives);

        return map;
    }
}
