package com.xhkjedu.smarking.utils;

import com.alibaba.fastjson.JSON;
import com.xhkjedu.smarking.model.stupaper.MsPaperStudentQuestionMerge;
import com.xhkjedu.utils.N_Utils;

import java.math.BigDecimal;
import java.util.Arrays;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;
import java.util.regex.Pattern;

/**
 * @author ywx
 * @className ClassUtil
 * @description
 * @date 2022/2/21 17:20
 **/
public class MarkingUtil {
    public static Map<String, String> uriMap = new ConcurrentHashMap<>();

    /**
     * 删除Html标签
     *
     * @param inputString
     * @return
     */
    public static String htmlRemoveTag(String inputString) {
        if (inputString == null)
            return null;
        String htmlStr = inputString; // 含html标签的字符串
        String textStr = "";
        java.util.regex.Pattern p_script;
        java.util.regex.Matcher m_script;
        java.util.regex.Pattern p_style;
        java.util.regex.Matcher m_style;
        java.util.regex.Pattern p_html;
        java.util.regex.Matcher m_html;
        try {
            //定义script的正则表达式{或<script[^>]*?>[\\s\\S]*?<\\/script>
            String regEx_script = "<[\\s]*?script[^>]*?>[\\s\\S]*?<[\\s]*?\\/[\\s]*?script[\\s]*?>";
            //定义style的正则表达式{或<style[^>]*?>[\\s\\S]*?<\\/style>
            String regEx_style = "<[\\s]*?style[^>]*?>[\\s\\S]*?<[\\s]*?\\/[\\s]*?style[\\s]*?>";
            String regEx_html = "<[^>]+>"; // 定义HTML标签的正则表达式
            p_script = Pattern.compile(regEx_script, Pattern.CASE_INSENSITIVE);
            m_script = p_script.matcher(htmlStr);
            htmlStr = m_script.replaceAll(""); // 过滤script标签
            p_style = Pattern.compile(regEx_style, Pattern.CASE_INSENSITIVE);
            m_style = p_style.matcher(htmlStr);
            htmlStr = m_style.replaceAll(""); // 过滤style标签
            p_html = Pattern.compile(regEx_html, Pattern.CASE_INSENSITIVE);
            m_html = p_html.matcher(htmlStr);
            htmlStr = m_html.replaceAll(""); // 过滤html标签
            textStr = htmlStr;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return textStr;// 返回文本字符串
    }

    /*
     * @Description 判断两个字符串的关系
     * @Date 2024/10/29 10:23:58
     * @Author WN
     * @Param [str1, str2]
     * @Return int
     **/
    public static int isEqualStringCheck(String str1,String str2){
        char[] charArray1 = str1.toCharArray();
        char[] charArray2 = str2.toCharArray();
        Arrays.sort(charArray1);
        Arrays.sort(charArray2);
        int eqResult = 0;//0代表不相等 1表示两个字符串相等 2代表字符串1包含字符串2
        if(Arrays.equals(charArray1, charArray2)){
            eqResult = 1;
        }

        if(Arrays.asList(charArray1).containsAll(Arrays.asList(charArray2))){
            eqResult = 2;
        }

        return eqResult;
    }

    /*
     * @Description 客观题自动批阅（扫描）
     * @Date 2024/10/29 10:34:54
     * @Author WN
     * @Param [stuQuestion 学生作答, queAnswer 试题答案]
     * @Return void
     **/
    public static void checkObjectiveQuestion(MsPaperStudentQuestionMerge stuQuestion,MsPaperStudentQuestionMerge queAnswer){
        String stuAnswer = stuQuestion.getStuanswer();
        Double stuScore = 0D;
        Integer timestamp = N_Utils.getSecondTimestamp();

        //学生答案为空时
        if(N_Utils.isEmpty(stuAnswer)){
            stuQuestion.setAnswered(0);
            stuQuestion.setStuanswer(null);
            stuQuestion.setStuscore(stuScore);
            stuQuestion.setChecked(2);
            stuQuestion.setFirstcime(timestamp);
            stuQuestion.setFirstccore(stuScore);
        }else{
            //如果试题进行了得分设置，则直接进行分值处理(分值设置0默认1送分题 20分题)
            Integer scoreset = queAnswer.getScoreset();
            if(N_Utils.isTrueInteger(scoreset)){
                if(scoreset == 1){
                    stuScore = queAnswer.getQscore();
                }
            }else{
                //未进行分值设置
                //学生答案不为空，试卷设置了正确答案
                if(queAnswer.getHasanswer()!=null && queAnswer.getHasanswer() == 1){
                    String qanswer = JSON.parseArray(queAnswer.getQanswer(), String.class).get(0);//试题正确答案
                    int ctype = queAnswer.getCtype();
                    Double qscore = queAnswer.getQscore();
                    if(N_Utils.isObjectiveQuestion(ctype)){
                        //客观题进行批阅
                        if(ctype == 1 || ctype == 4 || ctype == 5 || ctype == 6){
                            //单选题、判断题进行判断
                            if(stuAnswer.equalsIgnoreCase(qanswer)){
                                stuScore = qscore;
                            }
                        }else if(ctype == 2){
                            //多选题进行判断，
                            Double lostoption = queAnswer.getLoseoption();//少选分值
                            int isEqualStr = isEqualStringCheck(qanswer, stuAnswer);
                            if(isEqualStr == 1){
                                stuScore = qscore;
                            }else if(isEqualStr == 2){
                                stuScore = lostoption;//少选得分
                            }

                        }
                        stuQuestion.setChecked(2);
                        stuQuestion.setFirstcime(timestamp);
                    }

                }else{
                    stuQuestion.setChecked(0);
                }
            }
            stuQuestion.setStuscore(stuScore);
            stuQuestion.setAnswered(1);
        }
    }

    /**
     * 提供精确的加法运算。
     *
     * @param v1 被加数
     * @param v2 加数
     * @return 两个参数的和
     */
    public static double add(double v1, double v2) {
        BigDecimal b1 = new BigDecimal(Double.toString(v1));
        BigDecimal b2 = new BigDecimal(Double.toString(v2));
        return b1.add(b2).doubleValue();
    }
}
